local Build = select(4, GetBuildInfo())
local AddonName, AddonTable = ...
local L = AddonTable.Localize
LibStub:GetLibrary("BigLibTimer3"):Register(AddonTable)

if not SuperDuperSoundPackEngineAddonConfig then
	SuperDuperSoundPackEngineAddonConfig = {}
end

local SoundPacks = {}
local SoundPacksName = {}

local DefaultSoundPack = "SuperMarioWorldOfWarcraft"


local CLASS = select(2, UnitClass("player"))
local RACE = select(2, UnitRace("player")):upper():gsub("[^A-Z]", ""):gsub("SCOURGE", "UNDEAD")

local VARIABLES_CHECKED = nil
local SOUND_SUSPENDED = nil
local CHECKS_DISABLED = nil

local LOADING = true
local FIRSTLOAD = true
local SoundPacksLoaded = nil
local STARMUSIC = nil
local SWIMMINGMUSIC = nil
local FLYINGMUSIC = nil
local VEHICLEMUSIC = nil
local RESTINGMUSIC = nil
local MOUNTEDMUSIC = nil
local SPEEDMUISIC = nil
local MUSIC_DELAYED = nil
local InCombat = InCombatLockdown()
local MOUNTED = nil
local GROUPWIPED = nil
local BATTLEFIELDWINNER = nil
local WASINPVEINSTANCE = nil
local LAST_HIT_ENEMY_PLAYER_TARGET_GUID = nil
local LAST_ENEMY_PLAYER_TARGET_GUID = nil
local BOSSID = {}
local SHIELDBUFFS = {
	(GetSpellInfo(17--[[Power Word: Shield]])),
	(GetSpellInfo(642--[[Divine Shield]])),
	(GetSpellInfo(1463--[[Mana Shield]])),
	(GetSpellInfo(11426--[[Ice Barrier]])),
	(GetSpellInfo(7812--[[Sacrifice]])),
}
local FORMBUFFS = {
	(GetSpellInfo(1066--[[Aquatic Form]])),
	(GetSpellInfo(5487--[[Bear Form]])),
	(GetSpellInfo(768--[[Cat Form]])),
	(GetSpellInfo(9634--[[Dire Bear Form]])),
	(GetSpellInfo(33943--[[Flight Form]])),
	(GetSpellInfo(24858--[[Moonkin Form]])),
	(GetSpellInfo(40120--[[Swift Flight Form]])),
	(GetSpellInfo(783--[[Travel Form]])),
	(GetSpellInfo(65139--[[Tree of Life]])),
	(GetSpellInfo(2645--[[Ghost Wolf]])),
	(GetSpellInfo(59672--[[Metamorphosis]])),
}

function Register_SuperDuperSoundPackEngine_SoundPack(Name, Title, Description, Version)
	if type(Name) == "string" and Name ~= "" then
		if type(SoundPacks[Name]) ~= "table" then
			SoundPacks[Name] = {}
		end
		SoundPacks[Name].Title = Title
		SoundPacks[Name].Description = Description
		SoundPacks[Name].Version = Version
		local v = ""
		if Version and Version ~= "" then
			v = " "..ORANGE_FONT_COLOR_CODE..Version..FONT_COLOR_CODE_CLOSE
		end
		SoundPacks[Name].DisplayTitle = (Title or "")..v.."   "..GRAY_FONT_COLOR_CODE.."("..Name..")"..FONT_COLOR_CODE_CLOSE
		SoundPacksName[SoundPacks[Name].DisplayTitle] = Name
		return SoundPacks[Name]
	end
	return nil
end

local function SoundPackValue(Key)
	if SoundPacksLoaded then
		return SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack][Key]
	end
	return nil
end

local SHIELDBUFFS_SIZE = select("#", unpack(SHIELDBUFFS))
local function IsShield(buff)
	if buff then
		for i = 1, SHIELDBUFFS_SIZE do
			if buff == SHIELDBUFFS[i] then
				return true
			end
		end
	else
		for i = 1, SHIELDBUFFS_SIZE do
			if SHIELDBUFFS[i] and UnitBuff("player", SHIELDBUFFS[i]) then
				STARMUSIC = 1
				return true
			end
		end
		STARMUSIC = nil
	end
	return false
end

local FORMBUFFS_SIZE = select("#", unpack(FORMBUFFS))
local function IsForm(buff)
	if CLASS == "DRUID" or CLASS == "WARLOCK" or CLASS == "SHAMAN" then
		if buff then
			for i = 1, FORMBUFFS_SIZE do
				if buff == FORMBUFFS[i] then
					return true
				end
			end
		elseif GetShapeshiftForm() ~= 0 then
			return true
		end
	end
	return false
end


local function ContinueSound(...)
	local File = ...
	local count = select("#", ...)
	if count > 1 then
		File = select(math.random(1, count), ...)
	end
	if type(File) == "string" then
		PlaySoundFile(File)
	end
end

function AddonTable.Sound(...)
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED then
		local First = ...
		if type(First) == "table" then
			ContinueSound(unpack(First))
		else
			ContinueSound(...)
		end
	end
end


local LAST_MULTI_MUSIC_FILE_USED = nil

local function ContinueMusic(...)
	local File = ...
	local count = select("#", ...)
	if count > 1 then
		local number = math.random(1, count)
		File = select(number, ...)
		if type(File) ~= "string" or LAST_MULTI_MUSIC_FILE_USED == File then
			File = {...}
			tremove(File, number)
			File = select(math.random(1, count - 1), unpack(File))
		end
		LAST_MULTI_MUSIC_FILE_USED = File
		AddonTable:SetTimer("ResetMusic", SoundPackValue("reset_multi_track_music_delay_time") or 300, SoundPackValue("reset_multi_track_music_delay_time") or 300, AddonTable.Music, ...)
	else
		AddonTable:ClearTimer("ResetMusic")
	end
	if type(File) == "string" then
		PlayMusic(File)
	end
end

function AddonTable.Music(...)
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
		local First = ...
		if type(First) == "table" then
			ContinueMusic(unpack(First))
		else
			ContinueMusic(...)
		end
	end
end


local function SilenceMusic()
	AddonTable:ClearTimer("ResetMusic")
	if SoundPacksLoaded and not SOUND_SUSPENDED and not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED then
		PlayMusic("Interface\\AddOns\\"..AddonName.."\\silence.ogg")
	end
end


local function SetMusic()
	if AddonTable:GetTimer("Delay") <= 0 or not MUSIC_DELAYED then
		MUSIC_DELAYED = nil
		if UnitIsDead("player") and not UnitIsGhost("player") then
			SilenceMusic()
			AddonTable:ClearTimer("immunity")
		elseif AddonTable:IsTimer("immunity") and STARMUSIC and not SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED and SoundPackValue("immunity_buff_on_self") then
			SilenceMusic()
		elseif AddonTable:GetTimer("Reloading") <= 0 then
			if STARMUSIC and not SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED and SoundPackValue("immunity_buff_on_self") then
				SilenceMusic()
				if not AddonTable:IsTimer("immunity") then
					AddonTable:SetTimer("immunity", nil, SoundPackValue("immunity_buff_on_self_repeat_time") or 0, AddonTable.Sound, SoundPackValue("immunity_buff_on_self"))
				end
			else
				AddonTable:ClearTimer("immunity")
				if SPEEDMUISIC and SoundPackValue("music_targeted_boss_with_low_health") then
					AddonTable.Music(SoundPackValue("music_targeted_boss_with_low_health"))
				elseif not InCombat and SWIMMINGMUSIC and SoundPackValue("music_swimming") then
					AddonTable.Music(SoundPackValue("music_swimming"))
				elseif not InCombat and FLYINGMUSIC and SoundPackValue("music_flying") then
					AddonTable.Music(SoundPackValue("music_flying"))
				elseif not InCombat and VEHICLEMUSIC and SoundPackValue("music_in_vehicle") then
					AddonTable.Music(SoundPackValue("music_in_vehicle"))
				else
					if InCombat and SoundPackValue("music_pve_instance_in_combat") and ( select(2, IsInInstance()) == "party" or select(2, IsInInstance()) == "raid" ) then
						AddonTable.Music(SoundPackValue("music_pve_instance_in_combat"))
					elseif not InCombat and SoundPackValue("music_pve_instance") and ( select(2, IsInInstance()) == "party" or select(2, IsInInstance()) == "raid" ) then
						AddonTable.Music(SoundPackValue("music_pve_instance"))
					elseif InCombat and SoundPackValue("music_in_combat") then
						AddonTable.Music(SoundPackValue("music_in_combat"))
					elseif RESTINGMUSIC and SoundPackValue("music_resting") then
						AddonTable.Music(SoundPackValue("music_resting"))
					elseif MOUNTEDMUSIC and SoundPackValue("music_mounted") then
						AddonTable.Music(SoundPackValue("music_mounted"))
					else
						AddonTable.Music(SoundPackValue("music_regular"))
					end
				end
			end
		else
			SilenceMusic()
			AddonTable:ClearTimer("immunity")
		end
	else
		SilenceMusic()
	end
end


local function StopAllMusic(DELAY)
	if DELAY == 0 then return end
	SilenceMusic()
	AddonTable:ClearTimer("immunity")
	if DELAY then
		MUSIC_DELAYED = 1
		AddonTable:SetTimer("Delay", DELAY, nil, SetMusic)
	else
		MUSIC_DELAYED = nil
		AddonTable:ClearTimer("Delay")
	end
end

local SetMusicVariables = {
	"ENABLE_SOUND",
	"ENABLE_MUSIC",
	"ENABLE_MUSIC_LOOPING",
}
local DelayMusicVariables = {
	"Sound_NumChannels",
	"Sound_OutputDriverIndex",
	"Sound_OutputQuality",
	"ENABLE_REVERB",
	"ENABLE_SOFTWARE_HRTF",
	"ENABLE_DSP_EFFECTS",
	"ENABLE_HARDWARE",
}
hooksecurefunc("SetCVar", function(cvar, value, scriptCVar)
	for _, v in ipairs(SetMusicVariables) do
		if cvar == v or scriptCVar == v then
			if value == "1" then
				SetMusic()
			end
			return
		end
	end
	for _, v in ipairs(DelayMusicVariables) do
		if cvar == v or scriptCVar == v then
			StopAllMusic(math.max(3, AddonTable:GetTimer("Delay")))
			return
		end
	end
end)


local function GroupWiped()
	if not GROUPWIPED then
		local group
		local members
		if UnitInRaid("player") then
			members = GetNumRaidMembers()
			group = "raid"
		else
			if not UnitIsDeadOrGhost("player") then
				return
			end
			members = GetNumPartyMembers()
			group = "party"
		end
		for i = 1, members do
			if not UnitIsDeadOrGhost(group..i) and UnitIsConnected(group..i) then
				return
			end
		end
		GROUPWIPED = 1
		AddonTable:SetTimer("GroupWiped"..GetTime(), (SoundPackValue("group_wipe_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("group_wipe"))
		StopAllMusic((SoundPackValue("group_wipe_play_time") or 0) + (SoundPackValue("group_wipe_delay_time") or 0) + AddonTable:GetTimer("Delay"))
	end
end


local function TestGroupWipe()
	if not GROUPWIPED and ( UnitInRaid("player") or GetNumPartyMembers() > 0 or CHECKS_DISABLED ) then
		AddonTable:SetTimer("TestGroupWipe", 0.5, nil, GroupWiped)
	end
end



local function TestPower()
	if UnitCanAttack("player", "target") and CheckInteractDistance("target", 3) and ( not UnitIsDeadOrGhost("target") or UnitExists("targettarget") or UnitAffectingCombat("target") ) and (
		GetComboPoints("player") == 5
		or
		(
			UnitPowerType("player") == SPELL_POWER_RAGE
			and UnitPowerMax("player", SPELL_POWER_RAGE) > 0
			and UnitPower("player", SPELL_POWER_RAGE) == UnitPowerMax("player", SPELL_POWER_RAGE)
		)
	) then
		AddonTable.Sound(SoundPackValue("full_combo_points_or_rage"))
	else
		AddonTable:ClearTimer("FULLPOWER")
	end
end

local function CheckVariables()
	if not SuperDuperSoundPackEngineAddonConfig then
		SuperDuperSoundPackEngineAddonConfig = {}
	end
	VARIABLES_CHECKED = 1
end

local function LoadOptionsFrameSettings()
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_AllSoundsEnabled:SetChecked(not SOUND_SUSPENDED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Music:SetChecked(not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Sounds:SetChecked(not SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Jumping:SetChecked(not SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Swimming:SetChecked(not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Dmg:SetChecked(not SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_XP:SetChecked(not SuperDuperSoundPackEngineAddonConfig.XP_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_OnBuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.BUFF_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_OnDebuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.DEBUFF_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_BossDieing:SetChecked(not SuperDuperSoundPackEngineAddonConfig.BOSSDIEING_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_FullPower:SetChecked(not SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_DamageImmunityBuff:SetChecked(not SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED)
	SuperDuperSoundPackEngineAddonOptionsFrameOptions_Checks:SetChecked(not CHECKS_DISABLED)
end


local function ResetDefaults()
	wipe(SuperDuperSoundPackEngineAddonConfig)
	SOUND_SUSPENDED = nil
	CHECKS_DISABLED = nil
	AddonTable.Sound(SoundPackValue("slash_command_or_option_save"))
end


local function SlashHandler(msg)
	local msg = msg:lower()
	if msg:match("v") then
		print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version"))
	elseif msg:match("check") then
		if CHECKS_DISABLED then
			wipe(BOSSID)
			CHECKS_DISABLED = nil
			print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version")..":  "..L["Checks enabled."])
		else
			CHECKS_DISABLED = true
			print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version")..":  "..L["Checks disabled for this session!"])
		end
		AddonTable.Sound(SoundPackValue("slash_command_or_option_save"))
		LoadOptionsFrameSettings()
	elseif msg:match("sou") or msg:match("aud") or msg:match("mus") then
		if SOUND_SUSPENDED then
			SOUND_SUSPENDED = nil
			SetMusic()
			print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version")..":  "..L["Sound enabled."])
		else
			SOUND_SUSPENDED = true
			StopMusic()
			print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version")..":  "..L["Sound disabled for this session!"])
		end
		AddonTable.Sound(SoundPackValue("slash_command_or_option_save"))
		LoadOptionsFrameSettings()
	elseif msg:match("reset") or msg:match("clear") or msg:match("delete") or msg:match("erase") then
		ResetDefaults()
		LoadOptionsFrameSettings()
		print(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version")..": "..L["Settings have been reset!"])
	else
		InterfaceOptionsFrame_OpenToCategory(SuperDuperSoundPackEngineAddonOptionsFrameOptions)
	end
end
SlashCmdList.SuperDuperSoundPackEngineAddon = SlashHandler
SLASH_SuperDuperSoundPackEngineAddon1 = "/soundpack"
SLASH_SuperDuperSoundPackEngineAddon2 = "/superduper"
SLASH_SuperDuperSoundPackEngineAddon3 = "/sdspe"
SLASH_SuperDuperSoundPackEngineAddon4 = "/supermariowow"
SLASH_SuperDuperSoundPackEngineAddon5 = "/mariowow"
SLASH_SuperDuperSoundPackEngineAddon6 = "/smwow"
SLASH_SuperDuperSoundPackEngineAddon7 = "/mario"
SLASH_SuperDuperSoundPackEngineAddon8 = "/smw"



function OnUpdate()
	if not LOADING then
		if IsMounted() then
			if not MOUNTED then
				MOUNTED = 1
				AddonTable.Sound(SoundPackValue("mount"))
			end
		elseif MOUNTED then
			MOUNTED = nil
			AddonTable.Sound(SoundPackValue("dismount"))
		end
		if IsSwimming() then
			if not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED and GetUnitSpeed("player") > 0 and HasFullControl() then
				if not AddonTable:IsTimer("swim") then
					AddonTable:SetTimer("swim", 0, SoundPackValue("swim_repeat_time") or 0, AddonTable.Sound, SoundPackValue("swim"))
				end
			else
				AddonTable:ClearTimer("swim")
			end
			if not SWIMMINGMUSIC and not SPEEDMUISIC and not InCombat and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
				SWIMMINGMUSIC = 1
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				SetMusic()
			end
			if FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				if not SWIMMINGMUSIC and not SPEEDMUISIC and not InCombat then
					SetMusic()
				end
			end
		else
			AddonTable:ClearTimer("swim")
			if IsFlying() or UnitOnTaxi("player") then
				if not FLYINGMUSIC and not SPEEDMUISIC and not InCombat and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					FLYINGMUSIC = 1
					SWIMMINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					if not FLYINGMUSIC and not SPEEDMUISIC and not InCombat then
						SetMusic()
					end
				end
			elseif UnitInVehicle("player") then
				if not VEHICLEMUSIC and not SPEEDMUISIC and not InCombat and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					VEHICLEMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					RESTINGMUSIC = nil
					MOUNTEDMUSIC = nil
					if not VEHICLEMUSIC and not SPEEDMUISIC and not InCombat then
						SetMusic()
					end
				end
			elseif IsResting() then
				if not RESTINGMUSIC and not SPEEDMUISIC and not InCombat and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					RESTINGMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					MOUNTEDMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or MOUNTEDMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					MOUNTEDMUSIC = nil
					if not RESTINGMUSIC and not SPEEDMUISIC and not InCombat then
						SetMusic()
					end
				end
			elseif IsMounted() then
				if not MOUNTEDMUSIC and not SPEEDMUISIC and not InCombat and ( not UnitIsDead("player") or UnitIsGhost("player") ) then
					MOUNTEDMUSIC = 1
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					SetMusic()
				end
				if SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC then
					SWIMMINGMUSIC = nil
					FLYINGMUSIC = nil
					VEHICLEMUSIC = nil
					RESTINGMUSIC = nil
					if not MOUNTEDMUSIC and not SPEEDMUISIC and not InCombat then
						SetMusic()
					end
				end
			elseif SWIMMINGMUSIC or FLYINGMUSIC or VEHICLEMUSIC or RESTINGMUSIC or MOUNTEDMUSIC then
				SWIMMINGMUSIC = nil
				FLYINGMUSIC = nil
				VEHICLEMUSIC = nil
				RESTINGMUSIC = nil
				MOUNTEDMUSIC = nil
				if not SPEEDMUISIC and not InCombat then
					SetMusic()
				end
			end
		end
	end
end
local UpdateFrame = CreateFrame("Frame")
UpdateFrame.UpdateInterval = 0.05
UpdateFrame.TimeSinceLastUpdate = 0
UpdateFrame:SetScript("OnUpdate", function(self, elapsed)
	self.TimeSinceLastUpdate = self.TimeSinceLastUpdate + elapsed
	if self.TimeSinceLastUpdate >= self.UpdateInterval and not self.Running then
		self.Running = true
		self.TimeSinceLastUpdate = 0
		OnUpdate()
		self.Running = false
	end
end)
UpdateFrame:Show()

local function UnitIsBossUnit(unit)
	for i = 1, 4 do
		if UnitExists("boss"..i) and UnitIsUnit(unit, "boss"..i) then
			return true
		end
	end
	return false
end

local function IsBoss(unit)
	local unit = unit or "target"
	return UnitExists(unit) and not UnitPlayerControlled(unit) and ( tostring(UnitClassification(unit)):lower():match("boss") or UnitLevel(unit) == -1 or UnitIsBossUnit(unit) )
end

local function UpdateSoundPackDescription()
	if SoundPacksLoaded then
		SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDescriptionLabel:SetText(SoundPacks[SoundPacksName[UIDropDownMenu_GetText(SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDropDownMenu)]].Description or "")
	end
end

local function OnEvent(self, event, ...)
	if event == "PLAYER_REGEN_DISABLED" then
			
			GROUPWIPED = nil
			InCombat = 1
			SetMusic()
			
	elseif event == "PLAYER_REGEN_ENABLED" then
			
			SPEEDMUISIC = nil
			InCombat = nil
			SetMusic()
			
	elseif event == "COMBAT_LOG_EVENT_UNFILTERED" then
			
			local Event, sourceGUID, GUID, SpellName, EventType
			if Build >= 40200 then
				Event = select(2, ...)
				sourceGUID = select(4, ...)
				GUID = select(8, ...)
				SpellName = select(13, ...)
				EventType = select(15, ...)
			elseif Build >= 40100 then
				Event = select(2, ...)
				sourceGUID = select(4, ...)
				GUID = select(7, ...)
				SpellName = select(11, ...)
				EventType = select(13, ...)
			else
				Event = select(2, ...)
				sourceGUID = select(3, ...)
				GUID = select(6, ...)
				SpellName = select(10, ...)
				EventType = select(12, ...)
			end
			if Event == "PARTY_KILL" and ( sourceGUID == UnitGUID("player") or sourceGUID == UnitGUID("pet") ) then
				AddonTable.Sound(SoundPackValue("kill_shot"))
			elseif Event == "UNIT_DIED" then
				if BOSSID[GUID] then
					BOSSID[GUID] = nil
					AddonTable:SetTimer("BossDied"..GetTime(), (SoundPackValue("boss_died_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("boss_died"))
					StopAllMusic((SoundPackValue("boss_died_play_time") or 0) + (SoundPackValue("boss_died_delay_time") or 0) + AddonTable:GetTimer("Delay"))
				elseif LAST_HIT_ENEMY_PLAYER_TARGET_GUID == GUID then
					if AddonTable:GetTimer("Delay") <= 3 then
						AddonTable:SetTimer("player_kill"..GetTime(), (1 or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("helped_kill_a_player"))
						if not AddonTable:ReplaceTimer("Delay", (SoundPackValue("helped_kill_a_player_play_time") or 0) + (SoundPackValue("helped_kill_a_player_delay_time") or 0) + AddonTable:GetTimer("Delay")) then
							AddonTable:SetTimer("Delay", (SoundPackValue("helped_kill_a_player_play_time") or 0) + (SoundPackValue("helped_kill_a_player_delay_time") or 0) + AddonTable:GetTimer("Delay"))
						end
					end
				else
					if GUID == UnitGUID("player") then
						if AddonTable:GetTimer("Delay") <= 3 then
							AddonTable:SetTimer("PlayerDied"..GetTime(), (SoundPackValue("self_died_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("self_died"))
							StopAllMusic((SoundPackValue("self_died_play_time") or 0) + (SoundPackValue("self_died_delay_time") or 0) + AddonTable:GetTimer("Delay"))
						end
						STARMUSIC = nil
						SPEEDMUISIC = nil
					end
					TestGroupWipe()
				end
			elseif Event == "SPELL_AURA_APPLIED" and GUID == UnitGUID("player") then
				if EventType == "BUFF" then
					if IsForm(SpellName) then
						if not LOADING and not AddonTable:IsTimer("buffup") then
							AddonTable:SetTimer("buffup", 1)
							AddonTable.Sound(SoundPackValue("shape_shift"))
						end
					else
						if not SuperDuperSoundPackEngineAddonConfig.BUFF_DISABLED and not AddonTable:IsTimer("buffup") then
							AddonTable:SetTimer("buffup", 1)
							AddonTable.Sound(SoundPackValue("buff_on_self"))
						end
						if not SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED and not STARMUSIC and IsShield(SpellName) then
							STARMUSIC = 1
							if not MUSIC_DELAYED then
								SilenceMusic()
								if not AddonTable:IsTimer("immunity") then
									AddonTable:SetTimer("immunity", AddonTable:GetTimer("buffup"), SoundPackValue("immunity_buff_on_self_repeat_time") or 0, AddonTable.Sound, SoundPackValue("immunity_buff_on_self"))
								end
							elseif AddonTable:GetTimer("Delay") <= AddonTable:GetTimer("buffup") then
								StopAllMusic(AddonTable:GetTimer("buffup"))
							end
						end
					end
				elseif EventType == "DEBUFF" and not SuperDuperSoundPackEngineAddonConfig.DEBUFF_DISABLED and not AddonTable:IsTimer("powerdown") then
					AddonTable:SetTimer("powerdown", 1)
					AddonTable.Sound(SoundPackValue("debuff_on_self"))
				end
			elseif Event == "SPELL_AURA_REMOVED" and EventType == "BUFF" and GUID == UnitGUID("player") and STARMUSIC and IsShield(SpellName) then
				IsShield()
				if not STARMUSIC then
					SetMusic()
				end
			else
				if ( sourceGUID == UnitGUID("player") or sourceGUID == UnitGUID("pet") ) and ( ( GUID == UnitGUID("target") and UnitIsPlayer("target") and UnitFactionGroup("target") ~= UnitFactionGroup("player") ) or ( not UnitExists("target") and GUID == LAST_ENEMY_PLAYER_TARGET_GUID ) ) then
					LAST_HIT_ENEMY_PLAYER_TARGET_GUID = GUID
				end
				if not SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED and sourceGUID == UnitGUID("player") and GUID == UnitGUID("target") and UnitCanAttack("player", "target") then
					if Event == "SPELL_DAMAGE" then
						AddonTable.Sound(SoundPackValue("target_spell_damage_from_self"))
					elseif Event == "SWING_DAMAGE" then
						AddonTable.Sound(SoundPackValue("target_swing_damage_from_self"))
					elseif Event == "RANGE_DAMAGE" then
						AddonTable.Sound(SoundPackValue("target_range_damage_from_self"))
					end
				end
			end
			
	elseif event == "PLAYER_TARGET_CHANGED" then
			
			if UnitIsPlayer("target") and UnitFactionGroup("target") ~= UnitFactionGroup("player") then
				LAST_ENEMY_PLAYER_TARGET_GUID = UnitGUID("target")
			elseif UnitExists("target") then
				LAST_ENEMY_PLAYER_TARGET_GUID = nil
			end
			
	elseif event == "PLAYER_XP_UPDATE" then
			
			if not SuperDuperSoundPackEngineAddonConfig.XP_DISABLED then
				AddonTable.Sound(SoundPackValue("honor_or_xp_gained"))
			end
			
	elseif event == "COMBAT_TEXT_UPDATE" then
			
			if not SuperDuperSoundPackEngineAddonConfig.XP_DISABLED then
				if (select(1, ...) == "HONOR_GAINED" and select(2, ...) > 0) or (select(1, ...) == "FACTION" and select(3, ...) > 0) then
					AddonTable.Sound(SoundPackValue("honor_or_xp_gained"))
				end
			end
			
	elseif event == "PLAYER_ALIVE" then
			
			AddonTable:SetTimer("Reloading", 2, nil,
				function()
					if LOADING then
						MOUNTED = IsMounted()
						LOADING = nil
					end
					IsShield()
					SetMusic()
				end
			)
			TestGroupWipe()
			
	elseif event == "PLAYER_ENTERING_WORLD" then
			
			wipe(BOSSID)
			local inInstance, instanceType = IsInInstance()
			if instanceType ~= "party" and instanceType ~= "raid" then
				inInstance = nil
			end
			if FIRSTLOAD then
				FIRSTLOAD = nil
				if not SuperDuperSoundPackEngineAddonConfig.SoundPack or not SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack] then
					if SoundPacks[DefaultSoundPack] then
						SuperDuperSoundPackEngineAddonConfig.SoundPack = DefaultSoundPack
					else
						SuperDuperSoundPackEngineAddonConfig.SoundPack = next(SoundPacks)
					end
				end
				SoundPacksLoaded = not not SuperDuperSoundPackEngineAddonConfig.SoundPack
				UIDropDownMenu_Initialize(SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDropDownMenu, function(frame)
					for _, v in pairs(SoundPacks) do
						local info = {}
						info.text = v.DisplayTitle
						info.func = function(self)
							UIDropDownMenu_SetSelectedID(frame, self:GetID())
							UpdateSoundPackDescription()
						end
						UIDropDownMenu_AddButton(info)
					end
				end)
				AddonTable.Sound(SoundPackValue("start_up"))
			elseif not UnitIsDeadOrGhost("player") then
				if instanceType == "pvp" or instanceType == "arena" then
					AddonTable.Sound(SoundPackValue("enter_pvp_instance"))
				elseif inInstance ~= WASINPVEINSTANCE then
					AddonTable.Sound(SoundPackValue("enter_or_leave_pve_instance"))
				end
			end
			SilenceMusic()
			WASINPVEINSTANCE = inInstance
			if BATTLEFIELDWINNER then
				if BATTLEFIELDWINNER == 1 then
					AddonTable:SetTimer("arena_victory"..GetTime(), (SoundPackValue("arena_victory_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("arena_victory"))
					StopAllMusic((SoundPackValue("arena_victory_play_time") or 0) + (SoundPackValue("arena_victory_delay_time") or 0) + AddonTable:GetTimer("Delay"))
				elseif BATTLEFIELDWINNER == UnitFactionGroup("player") then
					AddonTable:SetTimer("battleground_victory"..GetTime(), (SoundPackValue("battleground_victory_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("battleground_victory"))
					StopAllMusic((SoundPackValue("battleground_victory_play_time") or 0) + (SoundPackValue("battleground_victory_delay_time") or 0) + AddonTable:GetTimer("Delay"))
				elseif BATTLEFIELDWINNER ~= 0 then
					AddonTable:SetTimer("lost_battleground"..GetTime(), (SoundPackValue("lost_battleground_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("lost_battleground"))
					StopAllMusic((SoundPackValue("lost_battleground_play_time") or 0) + (SoundPackValue("lost_battleground_delay_time") or 0) + AddonTable:GetTimer("Delay"))
				end
				BATTLEFIELDWINNER = nil
			end
			AddonTable:SetTimer("Reloading", 2, nil,
				function()
					if LOADING then
						MOUNTED = IsMounted()
						LOADING = nil
					end
					IsShield()
					SetMusic()
				end
			)
			TestGroupWipe()
			
	elseif event == "UPDATE_BATTLEFIELD_SCORE" then
			
			if not BATTLEFIELDWINNER and GetBattlefieldWinner() then
				if select(2, IsInInstance()) == "pvp" then
					BATTLEFIELDWINNER = PLAYER_FACTION_GROUP[GetBattlefieldWinner()]
				elseif select(2, IsInInstance()) == "arena" then
					for i=1,GetNumBattlefieldScores() do
						if GetBattlefieldScore(i) == UnitName("player") then
							if select(6, GetBattlefieldScore(i)) == GetBattlefieldWinner() then
								BATTLEFIELDWINNER = 1
							else
								BATTLEFIELDWINNER = 0
							end
							break
						end
					end
				end
			end
			
	elseif event == "ACHIEVEMENT_EARNED" then
			
			AddonTable:SetTimer("achievement_earned"..GetTime(), (SoundPackValue("achievement_earned_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("achievement_earned"))
			StopAllMusic((SoundPackValue("achievement_earned_play_time") or 0) + (SoundPackValue("achievement_earned_delay_time") or 0) + AddonTable:GetTimer("Delay"))
			
	elseif event == "CHAT_MSG_SKILL" then
			
			AddonTable.Sound(SoundPackValue("skill_increase"))
			
	elseif event == "PLAYER_LEVEL_UP" then
			
			AddonTable.Sound(SoundPackValue("level_up_ding"))
			AddonTable:SetTimer("PLAYER_LEVEL_UP"..GetTime(), (SoundPackValue("level_up_chime_delay_time") or 0) + AddonTable:GetTimer("Delay"), nil, AddonTable.Sound, SoundPackValue("level_up_chime"))
			StopAllMusic((SoundPackValue("level_up_chime_play_time") or 0) + (SoundPackValue("level_up_chime_delay_time") or 0) + AddonTable:GetTimer("Delay"))
			
	elseif event == "ADDON_LOADED" then
			
			if select(1, ...) == AddonName then
				CheckVariables()
			end
			
	end
	
	if not SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED and SoundPackValue("full_combo_points_or_rage") and not AddonTable:IsTimer("FULLPOWER") then
		AddonTable:SetTimer("FULLPOWER", 0, SoundPackValue("full_combo_points_or_rage_repeat_time") or 0, TestPower)
	end
	
	if not UnitIsDeadOrGhost("target") and ( ( not CHECKS_DISABLED and IsBoss("target") ) or ( CHECKS_DISABLED and UnitCanAttack("player", "target") and not UnitPlayerControlled("target") ) ) then
		BOSSID[UnitGUID("target")] = true
		if UnitAffectingCombat("target") then
			if not SPEEDMUISIC and InCombat and ( ( UnitHealth("target") / UnitHealthMax("target") ) * 100 ) <= 25 and not UnitIsDeadOrGhost("player") then
				if SuperDuperSoundPackEngineAddonConfig.BOSSDIEING_DISABLED then
					SPEEDMUISIC = 1
					SetMusic()
				elseif not AddonTable:IsTimer("Delay") then
					SPEEDMUISIC = 1
					AddonTable.Sound(SoundPackValue("targeted_boss_reached_low_health"))
					StopAllMusic((3 or 0))
				end
			end
		end
		if SPEEDMUISIC and ( not InCombat or not UnitAffectingCombat("target") or ( ( UnitHealth("target") / UnitHealthMax("target") ) * 100 ) > 25 ) then
			SPEEDMUISIC = nil
			SetMusic()
		end
	end
	
end
local EventFrame = CreateFrame("Frame")
EventFrame:RegisterEvent("PLAYER_REGEN_ENABLED")
EventFrame:RegisterEvent("PLAYER_REGEN_DISABLED")
EventFrame:RegisterEvent("ADDON_LOADED")
EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
EventFrame:RegisterEvent("PLAYER_ALIVE")
EventFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
EventFrame:RegisterEvent("PLAYER_LEVEL_UP")
EventFrame:RegisterEvent("PLAYER_XP_UPDATE")
EventFrame:RegisterEvent("COMBAT_TEXT_UPDATE")
EventFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
EventFrame:RegisterEvent("UPDATE_BATTLEFIELD_SCORE")
EventFrame:RegisterEvent("UNIT_COMBO_POINTS")
EventFrame:RegisterEvent("CHAT_MSG_SKILL")
EventFrame:RegisterEvent("ACHIEVEMENT_EARNED")
EventFrame:SetScript("OnEvent", OnEvent)


local function OnToggleGameMenu()
	AddonTable.Sound(SoundPackValue("toggle_game_or_options_menu"))
end
GameMenuFrame:HookScript("OnShow", OnToggleGameMenu)
GameMenuFrame:HookScript("OnHide", OnToggleGameMenu)
InterfaceOptionsFrame:HookScript("OnShow", OnToggleGameMenu)

local function OnJump()
	if IsSwimming() then
		if not SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED and HasFullControl() and not AddonTable:IsTimer("swim") then
			AddonTable:SetTimer("swim", SoundPackValue("swim_repeat_time") or 0)
			AddonTable.Sound(SoundPackValue("swim"))
		end
	elseif not SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED and HasFullControl() and not IsFlying() and not UnitInVehicle("player") and not AddonTable:IsTimer("jump") then
		AddonTable:SetTimer("jump", SoundPackValue("jump_max_repeat_time") or 0)
		if SoundPackValue("mounted_jump") and IsMounted() then
			AddonTable.Sound(SoundPackValue("mounted_jump"))
		elseif SoundPackValue("gnome_jump") and RACE == "GNOME" then
			AddonTable.Sound(SoundPackValue("gnome_jump"))
		elseif SoundPackValue("elf_jump") and ( RACE == "BLOODELF" or ( RACE == "NIGHTELF" and not UnitIsDeadOrGhost("player") ) ) and not IsForm() then
			AddonTable.Sound(SoundPackValue("elf_jump"))
		else
			AddonTable.Sound(SoundPackValue("regular_jump"))
		end
	end
end
hooksecurefunc("JumpOrAscendStart", OnJump)

local function SaveOptionsFrameSettings()
	if SuperDuperSoundPackEngineAddonOptionsFrameOptions_AllSoundsEnabled:GetChecked() then
		if SOUND_SUSPENDED then
			SOUND_SUSPENDED = nil
			SetMusic()
		end
	elseif not SOUND_SUSPENDED then
		SOUND_SUSPENDED = true
		StopMusic()
	end
	if SuperDuperSoundPackEngineAddonOptionsFrameOptions_Music:GetChecked() then
		if SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
			SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED = nil
			SetMusic()
		end
	elseif not SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED then
		SuperDuperSoundPackEngineAddonConfig.MUSIC_DISABLED = true
		StopMusic()
	end
	if SuperDuperSoundPackEngineAddonOptionsFrameOptions_DamageImmunityBuff:GetChecked() then
		if SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED and STARMUSIC and IsShield() then
			SetMusic()
		end
		SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED = nil
	else
		SuperDuperSoundPackEngineAddonConfig.IMMUNITY_DISABLED = true
		if AddonTable:ClearTimer("immunity") then
			SetMusic()
		end
	end
	if SuperDuperSoundPackEngineAddonOptionsFrameOptions_Checks:GetChecked() then
		if CHECKS_DISABLED then
			wipe(BOSSID)
			CHECKS_DISABLED = nil
		end
	else
		CHECKS_DISABLED = true
	end
	SuperDuperSoundPackEngineAddonConfig.SOUNDS_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_Sounds:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.JUMP_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_Jumping:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.SWIM_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_Swimming:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.DAMAGE_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_Dmg:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.XP_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_XP:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.BUFF_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_OnBuff:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.DEBUFF_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_OnDebuff:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.BOSSDIEING_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_BossDieing:GetChecked() or nil
	SuperDuperSoundPackEngineAddonConfig.FULLPOWER_DISABLED = not SuperDuperSoundPackEngineAddonOptionsFrameOptions_FullPower:GetChecked() or nil
	AddonTable.Sound(SoundPackValue("slash_command_or_option_save"))
end

local function LocalizeFontStrings(frame)
	for _, f in next, {frame:GetChildren()} do
		if f:GetObjectType() == "FontString" then
			f:SetFont(STANDARD_TEXT_FONT, select(2, f:GetFont()))
			f:SetText(L[f:GetText()])
		else
			for _, f in next, {f:GetRegions()} do
				if f:GetObjectType() == "FontString" then
					f:SetFont(STANDARD_TEXT_FONT, select(2, f:GetFont()))
					f:SetText(L[f:GetText()])
				end
			end
		end
	end
end

function SuperDuperSoundPackEngineAddonOptionsFrameParent_OnLoad(self)
	LocalizeFontStrings(self)
	_G[self:GetName().."TitleString"]:SetText(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version"))	
	self.name = select(2, GetAddOnInfo(AddonName))
	self.okay = function(self)
		if SoundPacksLoaded then
			SuperDuperSoundPackEngineAddonConfig.SoundPack = SoundPacksName[UIDropDownMenu_GetText(SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDropDownMenu)]
		end
	end
	self.refresh = function(self)
		if SoundPacksLoaded then
			UIDropDownMenu_SetSelectedValue(SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDropDownMenu, SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack].DisplayTitle)
			UIDropDownMenu_SetText(SuperDuperSoundPackEngineAddonOptionsFrameParentSoundPackDropDownMenu, SoundPacks[SuperDuperSoundPackEngineAddonConfig.SoundPack].DisplayTitle)
			UpdateSoundPackDescription()
		end
	end
	self.cancel = function(self)
		AddonTable.Sound(SoundPackValue("toggle_game_or_options_menu"))
	end
	InterfaceOptions_AddCategory(self)
end

function SuperDuperSoundPackEngineAddonOptionsFrameOptions_OnLoad(self)
	LocalizeFontStrings(self)
	_G[self:GetName().."TitleString"]:SetText(select(2, GetAddOnInfo(AddonName)).." "..GetAddOnMetadata(AddonName, "Version"))
	self.parent = select(2, GetAddOnInfo(AddonName))
	self.name = L["Options"]
	self.okay = function(self)
		SaveOptionsFrameSettings()
	end
	self.refresh = function(self)
		LoadOptionsFrameSettings()
	end
	self.default = function(self)
		ResetDefaults()
	end
	InterfaceOptions_AddCategory(self)
end

